/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// GUIWx.h: interface for the GUIWx class.
//
//////////////////////////////////////////////////////////////////////

#ifndef GUIWX_H
#define GUIWX_H


#include "GUI.h"
#include "wxIncludes.h"
#include <map>

class wxDC;
class wxTextCtrl;
class wxStatusBar;
class wxPen;

class GUIWx : public GUI  {
public:
	/** Parameters: the width and height of the window, zoom applied to simulator values. */
	GUIWx(int iniPanX, int iniPanY, float zoom, wxStatusBar *iniStatusBar);
	virtual ~GUIWx();

	void setDC(wxDC *iniDC);

	virtual void clear();

	/** Draws a circle. */
	void drawCircle(float x, float y, float r);

	/** Draws a line */
	void drawLine(float x1, float y1, float x2, float y2);

	/** Draws a rectangle with the given bottom left corner, and with the given size */
	void drawRectangle(float x, float y, float w, float h);
	void drawCappedRectangle(float x, float y, float l, float R, float alpha);
	void setBrushColor(Color color);
	void setPenColor(Color color);
	virtual void drawArrow(float x1, float x2, float y1, float y2);

	/** Draws a force vector, having r as origin and force as magnitude. */
	void drawForce(const Vector2& r, const Vector2& force);

	/** Draws a torque, having r as rotation point, alpha as starting angle and
	torque as magnitude.*/
		void drawTorque(const Vector2& r, const real alpha, const real torque);

	/** Outputs a text string. */
	void outText(const char* s);

	/** Outputs a text to the statusbar, at position pos. */
	void outTextStatusBar(const char* s, int pos=0);

	/** The device context. */
	wxDC *dc;
	
private:	

	/** The status bar. */
	wxStatusBar *statusBar;
};

#endif //GUIWX_H
