/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Graph.cpp: implementation of the Graph class.
//
//////////////////////////////////////////////////////////////////////

#include "Graph.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Graph::Graph(int width, int height, int bufferSize, int nSubGraphs, float min, float max):
	width(width), height(height), bufferSize(bufferSize), nSubGraphs(nSubGraphs),
	min(min), max(max){
	subGraphs=new GraphData*[nSubGraphs];
	for(int i=0;i<nSubGraphs;i++){
		subGraphs[i]=new GraphData(bufferSize);
	}

	bitmap.Create(width,height);
	dc.SelectObject(bitmap);
	dc.SetBackground(*wxWHITE_BRUSH);
}

Graph::~Graph(){
	for(int i=0;i<nSubGraphs;i++){
		delete subGraphs[i];
	}
	delete[] subGraphs;
}

void Graph::push(float value, int nSubGraph){
	subGraphs[nSubGraph]->push(value);
	if(value<min) min=value;
	if(value>max) max=value;
}

void Graph::drawBuffer(){
	dc.Clear();
	dc.SetBrush(*wxTRANSPARENT_BRUSH);
	dc.SetPen(*wxBLACK_PEN);
	dc.DrawRectangle(0,0, width, height);
	int effectiveWidth=width-1;
	int effectiveHeight=height-1;
	int dx=1;
	int dy=1;
	float zx=(float)effectiveWidth/bufferSize;
	float zy=(float)effectiveHeight/(max-min);

	GraphData* graphData;
	int i, g;
	int x, xOld, y, yOld;
	for(g=0;g<nSubGraphs;g++){
		graphData=subGraphs[g];
		if(g==0)	dc.SetPen(*wxRED_PEN);
		else dc.SetPen(*wxGREEN_PEN);
		if(graphData->dataSize){
			xOld=dx;
			yOld=height-dy-zy*(graphData->getValue(0)-min);
			for(i=1;i<graphData->dataSize;i++){
				x=dx+i*zx;
				y=height-dy-zy*(graphData->getValue(i)-min);
				dc.DrawLine(xOld,yOld,x,y);
				xOld=x; 
				yOld=y;
			}
		}
	}
	wxString s;
	s.Printf("%g",max);
	dc.DrawText(s,dx,dy);
	s.Printf("%g",min);
	dc.DrawText(s,dx,height-dy-15);

}

void Graph::draw(wxDC* outdc, int x, int y){
	drawBuffer();
	outdc->Blit(x,y,width,height,&dc,0,0);	
}