/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// SimulatorThread.h: interface for the SimulatorThread class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SIMULATORTHREAD_H
#define SIMULATORTHREAD_H

#include "wxIncludes.h"

class ThyrixMainFrame;
class World;

class SimulatorThread : public wxThread  {
public:
	SimulatorThread(ThyrixMainFrame *iniFrame);
	virtual ~SimulatorThread();

	// thread execution starts here
	virtual void* Entry();

	/** Called when the thread exits - whether it terminates normally or is
	stopped with Delete() (but not when it is Kill()ed!) */
	virtual void OnExit();

	void setTimeFactor(float newFactor);

	/* Compute the interval between frames, in ms, given a desired number of frames 
	per second. **/
	void setFramesPerSecond(int fps);
	
	void setPause();

	/** Advances the simulator with a timestep. Called directly by the frame, following a command
	event. */
	void step();

	World* world;

private:
	/** Sleeps to adjust the speed of the simulator to a predefined speed.*/
	void sleepIfAhead();
	
	/** The time factor between the speed of the simulator and real time. */
	float timeFactor;
		
	/** The time interval for displaying a frame, in ms.*/
	unsigned frameInterval;
	
	/** Indicates that the simulation is paused.*/
	bool paused;

	/** The time (real time, relative to the starting time of the simulator or a later 
	reference time) at which the next simulator timestep should be executed, in s. */
	float expectedTime;

	/** A timer used for syncronizing the simulator time at a desired speed factor. 
	Counts the time since the starting of the simulation or a later reference time. */
	wxStopWatch simulationTimer;

	/** A pointer to the frame on which it will draw. */
	ThyrixMainFrame *frame;
	

};

#endif //SIMULATORTHREAD_H
