/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// SpikeGraph.h: interface for the SpikeGraph class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SPIKEGRAPH_H__BB61CB4C_B0D5_4444_BFA1_4088183CC669__INCLUDED_)
#define AFX_SPIKEGRAPH_H__BB61CB4C_B0D5_4444_BFA1_4088183CC669__INCLUDED_

#include "assert.h"
#include <wx/dc.h>

/** Draws a spikegram (used to illustrate the activity of a spiking neural network). */
class SpikeGraph  {
public:
	SpikeGraph(int nNeurons, int bufferSize);
	virtual ~SpikeGraph();

	/** The size of the memory buffer of the graph  */
	int bufferSize;

	/** The actual size of the data that was already set. */
	int dataSize;

	int nNeurons;

	/** The index that marks the start of the data. Virtually, the new data values added to the graph
	are pushed at the end of the data stack, while old values are popped at the beginning. In practice,
	just the index that marks the start of the data moves, while the new data values are overwritten
	over the older ones. */
	int index;

	void push(int neuron, bool value){
		assert(neuron<nNeurons);
		if(dataSize<bufferSize){
			data[neuron][dataSize]=value;
		} else {
			data[neuron][(index+dataSize)%dataSize]=value;
		}
	}

	void advanceTime(){
		if(dataSize<bufferSize){
			dataSize++;
		} else {
			index++;
		}
	}
	
	bool getValue(int neuron, int i){
		assert(i<dataSize);
		assert(neuron<nNeurons);
		return data[neuron][(index+i)%dataSize];
	}

	void draw(wxDC* outdc, int x, int y, int h=1);

private:
	bool** data;


};

#endif // !defined(AFX_SPIKEGRAPH_H__BB61CB4C_B0D5_4444_BFA1_4088183CC669__INCLUDED_)
