/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "wxIncludes.h"
#include <wx/xrc/xmlres.h>
#include <wx/image.h>
#include <wx/menu.h>
#include <wx/toolbar.h>
#include <wx/metafile.h>
#include <wx/gdicmn.h>
#include "ThyrixApplication.h"
#include "ThyrixMainFrame.h"
#include "World.h"


//Used in a C++ implementation file to complete the declaration of a class that 
//has run-time type information.
//(?)
IMPLEMENT_CLASS(ThyrixMainFrame, wxFrame)

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(ThyrixMainFrame, wxFrame)
    //EVT_MENU(XRCID("file_menu_exit"), ThyrixMainFrame::OnQuit)
    //EVT_MENU(XRCID("help_menu_about"), ThyrixMainFrame::OnAbout)
    //EVT_PAINT(ThyrixMainFrame::onPaint)
    EVT_MOTION(ThyrixMainFrame::onMotion)
		EVT_LEFT_DOWN(ThyrixMainFrame::onLeftDown)
		EVT_LEFT_UP(ThyrixMainFrame::onLeftUp)
		EVT_RIGHT_DOWN(ThyrixMainFrame::onRightDown)
		EVT_RIGHT_UP(ThyrixMainFrame::onRightUp)
    EVT_CLOSE(ThyrixMainFrame::OnClose)
    //EVT_IDLE(ThyrixMainFrame::OnIdle)
    EVT_MENU_RANGE(ThyrixMainFrame::kMenuTimePause, 
                   ThyrixMainFrame::kMenuTimeMax,
                   ThyrixMainFrame::onSetSpeed)
    EVT_MENU_RANGE(ThyrixMainFrame::kMenuFps30,
                   ThyrixMainFrame::kMenuFps90,
                   ThyrixMainFrame::onSetFps)
    EVT_MENU(kMenuStep, ThyrixMainFrame::onStep)
		EVT_MENU(kMenuCapture, ThyrixMainFrame::onCapture)

END_EVENT_TABLE()

//frame constructor
ThyrixMainFrame::ThyrixMainFrame(const wxString& title, World* world,
		int bufferWidth, int bufferHeight, int zoom)
    : wxFrame((wxFrame *)NULL, -1, title, wxDefaultPosition, wxDefaultSize),
      thread(this),
      gui(0),
			world(world),
			bufferWidth(bufferWidth),
			bufferHeight(bufferHeight),
			zoom(zoom){
	thread.world=world;
	
	// set the frame icon
	SetIcon(wxICON(Thyrix));
	
	//set the background color of the frame
	SetBackgroundColour(*wxWHITE);

	/* load the menubar & toolbar
       please, find a way to load these from the executable, not from
       separate data (.xml) files.
       SetMenuBar(wxXmlResource::Get()->LoadMenuBar("main_menubar"));
       SetToolBar(wxXmlResource::Get()->LoadToolBar(this, "main_toolbar"));
    */
	
	// Set the buffer bitmap and the corresponding memory DC
	bitmap.Create(bufferWidth,bufferHeight);
	buffer.SelectObject(bitmap);
	buffer.SetBackground(*wxWHITE_BRUSH);
	buffer.Clear();
	
	CreateStatusBar(3);

	//create a toolbar with one button
	wxToolBar* toolbar = CreateToolBar(wxTB_TEXT | wxTB_NOICONS);
	wxBitmap dummy;
	toolbar->AddTool(kMenuStep, "Step", dummy);
	toolbar->AddTool(kMenuCapture, "Capture", dummy);
	toolbar->Realize();

	buildMenu();
	
	SetClientSize(bufferWidth,bufferHeight+5);
	thread.Create();
}

// frame destructor
ThyrixMainFrame::~ThyrixMainFrame() {
	thread.Delete();
	delete gui;
	gui=NULL;
}

void ThyrixMainFrame::buildMenu() {
	wxMenu *timeMenu = new wxMenu;
	timeMenu->AppendRadioItem(kMenuTimePause,   "Step by step");
	timeMenu->AppendRadioItem(kMenuTimeDiv10,"/10 \tslow time");
	timeMenu->AppendRadioItem(kMenuTimeX1,   "x1 \treal time");
	timeMenu->AppendRadioItem(kMenuTimeX3,   "x3 \ttime factor");
	timeMenu->AppendRadioItem(kMenuTimeX10,  "x10 \ttime factor");
	timeMenu->AppendRadioItem(kMenuTimeX30,  "x30 \ttime factor");
	timeMenu->AppendRadioItem(kMenuTimeX100, "x100 \ttime factor");
	timeMenu->AppendRadioItem(kMenuTimeMax,  "Maximum speed");
	timeMenu->Check(kMenuTimeX1, true);
	//timeMenu->Check(kMenuTimeMax, true);

	wxMenu *displayMenu = new wxMenu;
	displayMenu->AppendRadioItem(kMenuFps30,   "30 Frames Per Second");
	displayMenu->AppendRadioItem(kMenuFps60,   "60 FPS");
	displayMenu->AppendRadioItem(kMenuFps90,   "90 FPS");
	displayMenu->Check(kMenuFps30, true);

	wxMenuBar *menubar = new wxMenuBar;
	menubar->Append(timeMenu, "Speed");
	menubar->Append(displayMenu, "Display");
	SetMenuBar(menubar);
}

void ThyrixMainFrame::start() {
	gui = new GUIWx(0,bufferHeight,zoom,GetStatusBar());
	thread.Run();
}

void ThyrixMainFrame::onStep(wxCommandEvent& event) {
	thread.step();
}

void ThyrixMainFrame::onCapture(wxCommandEvent& event) {
	wxMetafileDC* dc=new wxMetafileDC("capture.wmf",bufferWidth,bufferHeight);
	paintDC(dc);
	wxMetafile* metafile=dc->Close();
	metafile->SetClipboard((int)(dc->MaxX() + 10), (int)(dc->MaxY() + 10) );
	delete metafile;
	delete dc;

}

void ThyrixMainFrame::onSetFps(wxCommandEvent& event) {
	switch(event.GetId()) {
	case kMenuFps30:
		thread.setFramesPerSecond(30);
		break;
	case kMenuFps60:
		thread.setFramesPerSecond(60);
		break;
	case kMenuFps90:
		thread.setFramesPerSecond(90);
		break;
	}
}

void ThyrixMainFrame::onSetSpeed(wxCommandEvent& event) {
    switch(event.GetId()) {
    case kMenuTimePause:
        thread.setPause();
        break;
    case kMenuTimeDiv10:
        thread.setTimeFactor(0.1f);
        break;
    case kMenuTimeX1:
        thread.setTimeFactor(1);
        break;
    case kMenuTimeX3:
        thread.setTimeFactor(3);
        break;
    case kMenuTimeX10:
        thread.setTimeFactor(10);
        break;
    case kMenuTimeX30:
        thread.setTimeFactor(30);
        break;
    case kMenuTimeX100:
        thread.setTimeFactor(100);
        break;
    case kMenuTimeMax:
        thread.setTimeFactor(1000);
        break;
    }
}

/*
void ThyrixMainFrame::OnIdle(wxIdleEvent& event) {
    event.RequestMore();
    
    for (int i = 0; i < 100; ++i) {
        simulator->advanceTime();
    }
    paint();
}
*/



void ThyrixMainFrame::OnClose(wxCloseEvent& event){
	//thread.Delete();
	Destroy();
}

/*
void ThyrixMainFrame::OnQuit(wxCommandEvent& event){
	// TRUE is to force the frame to close
	Close(TRUE);
}
*/

/*
void ThyrixMainFrame::OnAbout(wxCommandEvent& event){
	// called when help - about is picked from the menu or toolbar
	wxString msg;
	msg.Printf(_("Thyrix: a simulator for virtual environments\n(c) Arxia / Coneural 2003\n"));
	wxMessageBox(msg, _("About Thyrix"), wxOK | wxICON_INFORMATION, this);
}
*/

/*
void ThyrixMainFrame::onPaint(wxPaintEvent& event){
	//If no DC is created, the program does not stop. From the WX documentation (wxPaintEvent):
	//In a paint event handler, the application must always create a wxPaintDC object, 
	//even if you do not use it. Otherwise, under MS Windows, refreshing for this and 
	//other windows will go wrong.

	wxPaintDC dc(this); 
	//dc.Blit(0,50,600,400,&buffer,0,0);
}
*/

void ThyrixMainFrame::paint(){
	wxClientDC dc(this);
	//here you can chose, to paint either directly to screen,
	//or paint to a memoryDC & copy to screen (to avoid flicker).
	//paintDC(&dc);
	paintDC(&buffer);
	dc.Blit(0,30,bufferWidth,bufferHeight,&buffer,0,0);
}

/*
void ThyrixMainFrame::onEraseBackground(wxEraseEvent& event){
}
*/

void ThyrixMainFrame::onMotion(wxMouseEvent& event){
	if(event.LeftIsDown()){
		world->setMouseCoordinates(gui->inverseMapX(event.GetX()), gui->inverseMapY(event.GetY()));
	}
}

void ThyrixMainFrame::onLeftDown(wxMouseEvent& event){
	float x=gui->inverseMapX(event.GetX());
	float y=gui->inverseMapY(event.GetY());
	world->onMouseLeftDown(x,y);
	//simulator->applyMouseForce(x,y);
	//simulator->setMouseCoordinates(x,y);
}

void ThyrixMainFrame::onLeftUp(wxMouseEvent& event){
	float x=gui->inverseMapX(event.GetX());
	float y=gui->inverseMapY(event.GetY());
	world->onMouseLeftUp(x,y);
	//simulator->unsetMouseForce();
}

void ThyrixMainFrame::onRightDown(wxMouseEvent& event){
	float x=gui->inverseMapX(event.GetX());
	float y=gui->inverseMapY(event.GetY());
	world->onMouseRightDown(x,y);
}

void ThyrixMainFrame::onRightUp(wxMouseEvent& event){
	float x=gui->inverseMapX(event.GetX());
	float y=gui->inverseMapY(event.GetY());
	world->onMouseRightUp(x,y);
}


void ThyrixMainFrame::paintDC(wxDC* dc){
	//dc->SetOptimization(true);
	dc->BeginDrawing();
	dc->Clear();
	gui->setDC(dc);
	world->draw(gui);
	dc->EndDrawing();
}
