/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "IunctusSimulator.h"
#include "Border.h"
#include "Circle.h"




IunctusSimulator::IunctusSimulator() {

	real width=6, height=4;
	//set the borders of the simulator space
	registerObject(new Border(0, height/2, 1, 0, "left border"));
	registerObject(new Border(width, height/2, -1, 0, "right border"));
	registerObject(new Border(width/2, 0,	0, 1, "bottom border"));
	registerObject(new Border(width/2, height, 0,  -1, "upper border"));

	/** r, x, y. */
	float circles[6][3] = {
		{0.1f, 5.0f, 3.0f},
		{0.2f, 0.4f, 0.6f},
		{0.1f, 0.8f, 0.9f},
		{0.2f, 1.7f, 2.8f},
		{0.25f, 4.7f, 2.5f},
		{0.1f, 3.5f, 1.0f}
	};
	
	//add the agent
	registerObject(agent=new Iunctus(2.0, 0.8, "Iunctus"));

	//add balls

	char s[10];

	
	for (unsigned i = 0; i < 6; ++i) {
		sprintf(s,"ball %d",i+1);
		registerObject(new Circle(circles[i][0], circles[i][1], circles[i][2], s));
	}
	

	
	registerObject(new CappedRectangle(0.3f,  0.1f, 3.0f, 2.2f, 10*degrees, "capsule 1"));
	registerObject(new CappedRectangle(0.6f, 0.05f, 1.0f, 3.2f, 60*degrees, "capsule 2"));
	registerObject(new CappedRectangle(0.5f, 0.12f, 5.0f, 1.2f, 80*degrees, "capsule 3"));
	registerObject(new CappedRectangle(0.4f, 0.08f, 2.3f, 3.3f, 160*degrees, "capsule 4"));
	ComposedPhysicalObject* c=new ComposedPhysicalObject(2.0f, 2.5f, 0, "double ball");
	c->addObject(new Circle(0.1),-0.1,0,0);
	c->addObject(new Circle(0.1),0.1,0,0);
	registerObject(c);
}

IunctusSimulator::~IunctusSimulator(){
}

void IunctusSimulator::detectContacts(){
	Simulator::detectContacts();
	/*
	The eye is not involved into contact detection by its parent, the agent.
	We can do this by either:
	1. Changing the bounding box of the agent to include the bounding box of the eye, thus becoming infinite,
	and including eye contact detection with agent contact detection (as for Spherus).
	This is innefficient because we will then need to verify in detail the contact of all agent links
	with all objects.
	2. Changing the method of contact detection in Simulator::detectContacts by performing culling with 
	bounding boxes at the level of primitives. Thus, the eye will be treated separately of the agent.
	Then, we will have to ensure that there is no contact checking between successive articulated links.
	This is best but requires distancing slightly the links and coding global changes in the implementation 
	of contact detection (to do...).
	3. The method implemented below: we call here 'manually' contact detection for the eye with all objects.
	*/
	PhysicalObjectPVector::iterator objectsI, objectsEnd=objects.end();
	for(objectsI=objects.begin(); objectsI!=objectsEnd; ++objectsI){
			agent->eye->detectContacts(*objectsI, &contacts);
	}

}