/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Border.h: interface for the Border class.
//
//////////////////////////////////////////////////////////////////////

#ifndef BORDER_H
#define BORDER_H

#include "PhysicalObject.h"

class Simulator;

/** A border object: a line that delimitates a semiplane in the simulation space. The 
line has infinite length; it is defined by one point belonging to the line and
the normal to this line, oriented towards the interior (the zone where the simulated objects evolve). 
The location of the reference point r on the line does not matter.*/

class Border : public PhysicalObject  {
public:
	/** Constructor, sets the position and the normal. */
	Border(real x, real y, real normalX, real normalY, std::string label="");
	/** Destructor, does nothing. */
	virtual ~Border();

  virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) {
		return object->detectContacts(this, contacts);
  }

  /** No contact is possible between two border objects. */
  virtual bool detectContacts(Border* object, GlobalContactInfoVector* contacts) { return false; }

	/** Does nothing (the border is fixed). Overrides method defined in PhysicalObject. */
	virtual void computeDerivativesWithoutContacts(ContactSolver* contactSolver) {}

	virtual void integrate(const Integrator &integrator) {}
	virtual void rollback() {}

	void fillContactMatrix(ContactSolver* contactSolver, ContactInfo* contact) {}
	void draw(GUI *gui)	{}

	/** The normal to the border. */
	Vector2 normal;
};

#endif //BORDER_H
