/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ContactInfo.h: interface for the ContactInfo class.
//
//////////////////////////////////////////////////////////////////////

#ifndef CONTACTINFO_H
#define CONTACTINFO_H

#include "Vector2.h"
#include <vector>

enum ContactType{
	contactTypeForce,
	contactTypeForceParallel,
	contactTypeTorque
};

class PhysicalObject;

class ContactInfo  {
public:
	/** Constructs a torque contact. */
	ContactInfo(PhysicalObject* object, int sigma);

	/** Constructs a force contact. Initializes information, and also 
	adds the contact to the object (or its parent,	if exists - for 
	composed objects)*/
	ContactInfo(PhysicalObject* object, int sigma, const Vector2& p, 
                const Vector2& n, int iSensor=-1);

	/** Constructs a force contact for parallel contact of capsules. */
	ContactInfo(PhysicalObject* object, int sigma, const Vector2& p, 
                const Vector2& n, int iSensor, int iSensor2);

	virtual ~ContactInfo();

	/** The sign of the participation of this object to the contact */
	int sigma;

	/** The type of the contact: force or torque. */
	ContactType type;

	//For force contacts:
	/** The position of the contact point relative to the object, 
	in the laboratory reference system. */
	Vector2 p;
	/** The direction of the contact force (the normal to the contact plane), in the
	laboratory reference system. */
	Vector2 n;
	/** The cross product p x n (having only a component on the z axis). */
	real pxn;


	/** The index of the tactile sensor, or of the ends of a range of tactile sensors 
	(for parallel contact of capsules) affected by the contact, if any. */
	int iSensor, iSensor2;

	/** The object that processes the contact (for objects that are part of a composed object,
	the composed object processes the contact). */
	PhysicalObject* parentObject;
	
	/** The object to which the contact belongs. */
	PhysicalObject* object;

	/** The modulus of the contact force. */
	real force;

	/** The index of this contact in the global contact vector. */
	int alpha;

private:
	void setupObject();
};

typedef std::vector<ContactInfo*> ContactInfoPVector;

#endif //CONTACTINFO_H
