/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef CONTACTSOLVER_H
#define CONTACTSOLVER_H

#include "ThyrixParameters.h"
#include "GlobalContactInfo.h"

enum ContactSet {
	contactSetC,	//contact
	contactSetNC,	//non-contact
	contactSetI		//ignored (redundant constraint, f=0)
};

class ContactSolver {

public:
	ContactSolver(GlobalContactInfoVector* contacts);
	virtual ~ContactSolver();

	/** Resize the data members to allow a greater number of contacts, if 
	needed; set to zero contactMatrix, contactVector, contactForces. */
	void init();

	/** Computes the contact forces, using the Baraff algorithm. */
	void computeContacts();

	/** Computes the contact forces with a Gauss-Seidel like method (re)discovered by Preda. */
	void computeContactsPreda();

	/** Uploads the computed forces into simulator contacts.*/
	void uploadForces();

	/** The A matrix of the Baraff algorithm. */
	real** contactMatrix;

	/** The b vector of the Baraff algorithm. */
	real* contactVector;

	/** A temporary vector for system solving. */
	real* tempVector1;
	real* tempVector2;
	real** tempMatrix;
	
private:
	real maxStep(int d, int *sIndex);
	bool driveToZero(int d);
	void initForcesVelocities();

	/** A pointer to the contacts list. */
	GlobalContactInfoVector* contacts;

	/** The current number of contacts. */
	int nContacts;

	/** The maximum number of contacts that fit in the current contact data structures. */
	int nContactsMax;
	
	/** A temporary matrix used in the Baraff algorithm for holding the A_CC. */
	real** contactMatrixTemp;

	/** The f vector of the Baraff algorithm. */
	real* contactForces;
	
	/** A temporary vector used in the Baraff algorithm. */
	real* contactVectorTemp;
	
	/** The delta f vector of the Baraff algorithm */
	real* contactDForces;

	/** The a vector of the Baraff algorithm. */
	real* contactVelocities;

	/** A vector which holds the set to which a contact belongs (C or NC) in the Baraff algorithm */
	ContactSet* contactSets;

	/** A index vector used in the Baraff algorithm to point to C contacts. */
	int* contactIndexC;

	/** A index vector used in the Baraff algorithm to change the order of contacts during processing */
	//int* contactIndex;

	/** A temporary index vector used in the Baraff algorithm for system solving. */
	int* contactIndexTemp;

	/** Counts the number of s=0 cycles for each contact */
	int* zeros;

};

#endif //CONTACTSOLVER_H
