/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// GUI.h: interface for the GUI class.
//
//////////////////////////////////////////////////////////////////////

#ifndef GUI_H
#define GUI_H

#include "Vector2.h"
#include "Color.h"

/* Provides an interface for the interaction of the simulator with a GUI. Can be specialized
for use under wxWidgets, MFC, etc. It is used for graphically illustrating the state of the
simulator on a display. **/
class GUI  {
public:
	GUI(float iniPanX=0, float iniPanY=400, float zoom=100.0f, int signX = 1, int signY = -1);
	virtual ~GUI();

	static const Color colorTransparent;
	static const Color colorWhite;
	static const Color colorBlack;

	void setZoom(float newZoom);

	/** Erases everything. */
	virtual void clear() = 0;

	virtual void drawCircle(float x, float y, float r) = 0;
	void drawCircle(const Vector2& r, float R) { drawCircle((float)r.x, (float)r.y, R);	}

	virtual void drawLine(float x1, float y1, float x2, float y2) = 0;
	void drawLine(const Vector2& r1, const Vector2& r2){
		drawLine((float)r1.x,(float)r1.y,(float)r2.x,(float)r2.y);
	}
	void drawLine(const Vector2& r1, float x2, float y2){
		drawLine((float)r1.x,(float)r1.y,x2,y2);
	}

	/** Draws a rectangle with the given bottom left corner, and with the given size. */
	virtual void drawRectangle(float x, float y, float w, float h) = 0;
	
	virtual void drawCappedRectangle(float x, float y, float l, float R, float alpha) = 0;
	virtual void setBrushColor(Color color) = 0;
	virtual void setPenColor(Color color) = 0;
	virtual void drawArrow(float x1, float x2, float y1, float y2) = 0;

	/** Outputs a text string. */
	virtual void outText(const char* s) = 0;

	/** Outputs a text to the statusbar, at position pos. */
	virtual void outTextStatusBar(const char* s, int pos) = 0;

	/** Draws a force vector, having r as origin and force as magnitude. */
	virtual void drawForce(const Vector2& r, const Vector2& force) = 0;

	/** Draws a torque, having r as rotation point, alpha as starting angle and
	torque as magnitude.*/
	virtual void drawTorque(const Vector2& r, const real alpha, const real torque) = 0;
	
	float mapX(float x) {
		//return panX + floor(zoomX * x+0.5);
		return panX + zoomX * x;
	}

	float mapY(float y) {
		//return panY + floor(zoomY * y+0.5);
		return panY + zoomY * y;
	}

	float inverseMapX(float x){
		return (float)(x-panX)/zoomX;
	}

	float inverseMapY(float y){
		return (float)(y-panY)/zoomY;
	}

	float mapLen(float len) {
		//mapLen is usually used for circle's ray, so it counts twice :)
		return zoom * len;
	}

	float panX;
	float panY;
	float zoom;
	int signX;
	int signY;
	float zoomX;
	float zoomY;
};


#endif //GUI_H
