/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef GRAPHICALOBJECT_H
#define GRAPHICALOBJECT_H

#include "GUI.h"

/** Base class for all objects that can be drawn (including physical objects, but eventually also 
elements like markers, contacts, forces, etc.). It is characterized by an outline color and a fill color. */

class GraphicalObject{
public:
	GraphicalObject(Color outlineColor = GUI::colorBlack, Color fillColor = GUI::colorTransparent);

	/** Destructor, does nothing. */
	virtual ~GraphicalObject();
	
	/** Draw the object to the graphical user interface. */
	virtual void draw(GUI *gui) = 0;

	/** Sets outline color. */
	virtual void setOutlineColor(Color color) { outlineColor=color;	}

	/** Sets fill color. */
	virtual void setFillColor(Color color) { fillColor=color;	}
	
	/** Sets both outline and fill color to the same color. */
	void setColor(Color color){
		setOutlineColor(color);
		setFillColor(color);
	}

	/** The outline color of the object. */
	Color outlineColor;

	/** The fill color of the object. */
	Color fillColor;
};

#endif //GRAPHICALOBJECT_H 
