/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Matrix3.h: interface for the Matrix3 class.
//
//////////////////////////////////////////////////////////////////////

#ifndef MATRIX3_H
#define MATRIX3_H

#include "Vector3.h"

/** A 3x3 matrix. */

class Matrix3  {

public:
	/** Default constructor, sets the matrix to 0.*/
	Matrix3(){
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++)
				matrix[i][j]=0.0;
	}
	virtual ~Matrix3();
	
	/** The placeholder for the data. */
	real matrix[3][3];
	
	/** Sets all matrix elements to 0. */
	void setToZero(){
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++)
				matrix[i][j]=0.0;
	}

	/** Sets element (i,j) to value value; i,j are numbered starting from 1 (i,j=1,2,3). */
	void setElement(int i, int j, real value)		{
		assert(i>=1 && i<=3 && j>=1 && j<=3);
		matrix[i-1][j-1]=value;
	}

	/** Sets row i of the matrix to (v1, v2, v3); i is numbered starting from 1 (i=1,2,3). */
	void setRow(int i, real v1, real v2, real v3) {
		assert(i>=1 && i<=3);
		matrix[i-1][0]=v1;
		matrix[i-1][1]=v2;
		matrix[i-1][2]=v3;
	}

	/** Sets column i of the matrix to v; i is numbered starting from 1 (i=1,2,3). */
	void setColumn(int i, const Vector3& v){
		assert(i>=1 && i<=3);
		matrix[0][i-1]=v.x;
		matrix[1][i-1]=v.y;
		matrix[2][i-1]=v.z;
	}

	/** Gets element (i,j) of the matrix; i,j are numbered starting from 1 (i,j=1,2,3). */
	real getElement(int i, int j) {
		assert(i>=1 && i<=3 && j>=1 && j<=3);
		return matrix[i-1][j-1];
	}

	/** Gets the determinant of the matrix. */
	real getDeterminant();

	/** Gets row i; i is numbered starting from 0 (i=0,1,2). */
	real* operator[](int i);


	/** Addition to another matrix. */
	Matrix3& operator+=(const Matrix3& m){
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++)
				matrix[i][j]+=m.matrix[i][j];
		return *this;
	}
	
	

};

#endif //MATRIX3_H
