/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// SymmetricMatrix3.h: interface for the SymmetricMatrix3 class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SYMMETRICMATRIX3_H
#define SYMMETRICMATRIX3_H

#include "Matrix3.h"

/** A class for symmetric 3x3 matrices. Only the upper half of the matrix should be used. The data storage
is still a matrix, for convenient access. */

class SymmetricMatrix3 : public Matrix3{
public:
	SymmetricMatrix3()					{}
	virtual ~SymmetricMatrix3()		{}

	/** Sets the elements from the upper half of the matrix. */
	void setElements(		real v1,	real v2,	real v3,
																real v4,	real v5,
																					real v6){
		matrix[0][0]=v1; matrix[0][1]=v2; matrix[0][3]=v3;
		matrix[1][1]=v4; matrix[1][2]=v5; 
		matrix[2][2]=v6;
	}

	/** Gets the determinant of the matrix, based on the upper half elements only. */
	real getDeterminant() const {
		return	matrix[0][0]*matrix[1][1]*matrix[2][2]+
					2*matrix[0][1]*matrix[0][2]*matrix[1][2]
					-sqr(matrix[1][2])*matrix[0][0]
					-sqr(matrix[0][2])*matrix[1][1]
					-sqr(matrix[0][1])*matrix[2][2];
	}

	/** Sets the lower half of the matrix to correspond to the upper half. */
	void mirror(){
		for(int i=0;i<2;i++)
			for(int j=i+1;j<3;j++)
				matrix[j][i]=matrix[i][j];
	}

	/** The + operator is restricted to the upper half of the matrix. */
	SymmetricMatrix3& operator+=(const SymmetricMatrix3& m){
		for(int i=0;i<3;i++)
			for(int j=i;j<3;j++)
				matrix[i][j]+=m.matrix[i][j];
		return *this;
	}

};

#endif //SYMMETRICMATRIX3_H
