/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// SystemSolver.h: interface for the SystemSolver class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SYSTEMSOLVER_H
#define SYSTEMSOLVER_H

#include "ThyrixParameters.h"

/** Functions for solving linear systems of equations. After Numerical Recipes (Press et al.) */

class SystemSolver  {
public:
	SystemSolver();
	virtual ~SystemSolver();

	/** Given a square matrix a of dimension n, the routine replaces it by the LU decomposition of a 
	rowwise permutation of itself. The original a matrix is destroyed
	and replaced with the output. index[0..n-1] is an output vector that records the row 
	permutation. tempVector is a temporary vector, it may be preallocated to a size at least n. */
	static void luDecompose(real** a, int n, int* index, real* tempVector);

	/** Solves the set of equations a x=b. a should previously be decomposed with luDecompose. index
	is the vector provided by luDecompose. b is destroyed and replaced with the solution vector. */
	static void luSubstitute(real** a, int n, int* index, real* b);

	/** Singular value decomposition of square matrix a of dimension n as u w vT. The data in
	the original matrix is destroyed, and is replaced with u.*/
	static void svDecompose(real** a, int n, real* w, real** v, real* tempVector);

	/** Solves the set of equations u w vT x=b. u, w, v should previously be generated with svDecompose. 
	w is replaced with the solution vector. */
	static void svSubstitute(real** u, int n, real* w, real** v, real* b, real* tempVector);

	

};

#endif //SYSTEMSOLVER_H
