/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Vector3.h: interface for the Vector3 class.
//
//////////////////////////////////////////////////////////////////////

#ifndef VECTOR3_H
#define VECTOR3_H

#include "ThyrixParameters.h"
#include <assert.h>

/** A 3D vector. */

class Vector3  {
public:
	/** The default constructor, sets the vector to 0. */
	Vector3()											{ x=0.0; y=0.0; z=0.0;	}
	/** Constructor. */
	Vector3(real x, real y, real z)				{ this->x=x; this->y=y; this->z=z; }
	/** Destructor; does nothing. */
	virtual ~Vector3()								{}

	real x;
	real y;
	real z;

	/** Sets the vector to 0. */
	void setToZero()									{ x=0.0; y=0.0; z=0.0;	}

	/** Sets x, y and z components. */
	void setXYZ(real x, real y, real z)			{ this->x=x; this->y=y; this->z=z; }

	/** Sets element i to value value; i is numbered starting from 1 (i=1,2,3). */
	void setElement(int i, real value){
		assert(i>=1 && i<=3);
		if (i==1)
			x=value;
		else if (i==2)
			y=value;
		else
			z=value;
	}
	
	/** Gets the module of the vector: sqrt(x^2+y^2+z^2). */
	real getModule()									{ return sqrt(x*x+y*y+z*z); }

	/** Gets the squared module of the vector: x^2+y^2+z^2. */
	real getSquaredModule()							{ return x*x+y*y+z*z; }

	/** Rotates the spatial part of the vector with angle alpha. 
	If the vector is initially expressed in a reference
	system rotated with alpha relative to the laboratory reference system, after rotation the vector
	will be expressed in the laboratory reference system. */
	void rotate(real alpha) {
		real yOld=y;
		real c=cos(alpha);
		real s=sin(alpha);
		y=y*c-z*s;
		z=yOld*s+z*c;
	}
	

	/** Gets element i; i is numbered starting from 0 (i=0,1,2). */
	real operator[](int i){
		assert(i>=0 && i<=2);
		if (i==0)
			return x;
		else if (i==1)
			return y;
		else return z;
	}

	Vector3& operator+=(const Vector3& v)	{ x+=v.x; y+=v.y; z+=v.z; return *this; }
	Vector3& operator-=(const Vector3& v)	{ x-=v.x; y-=v.y; z-=v.z; return *this; }
	Vector3& operator*=(const real r)		{ x*=r; y*=r; z*=r; return *this; }
	Vector3& operator/=(const real r)		{ assert(r!=0.0); x/=r; y/=r; z/=r; return *this; }

};

#endif //VECTOR3_H
