/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef PROXIMITYSENSOR_H
#define PROXIMITYSENSOR_H

#include "PhysicalObject.h"

/** A visual sensor, composed of unit sensors (pixels) arranged on a circular
segment. It currently has infinite range.
The sensor is attached to an object, at relative position relativeR and relative angle
relativeAlpha. The view angle is centered around the orientation of the sensor. 
*/

class VisualSensor : public PhysicalObject  {
public:
	/** Constructor. \param parent is the object is is attached to, at relative coordinates
	\param relativeX and \param relativeY and relative angle \relativeAlpha. The view angle should be 
	between 0..2Pi. */
	VisualSensor(PhysicalObject* parent, real relativeX, real relativeY, 
		real relativeAlpha, real R, real viewAngle, unsigned int nSensors,
		std::string label="");
	virtual ~VisualSensor();


	/** Redirects contact detection.*/
	virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) { 
		return object->detectContacts(this, contacts);
	}
	
	/** Tests for contacts between this object and a border. */
	bool detectContacts(Border* border, GlobalContactInfoVector* contacts){
		return false;
	}

	/** Tests for contacts between this object and a circle. */
	bool detectContacts(Circle* circle,  GlobalContactInfoVector* contacts);

	/** Tests for contacts between this object and a capsule. */
	bool detectContacts(CappedRectangle* capsule,  GlobalContactInfoVector* contacts);


	virtual void draw(GUI *gui);

private:

	/** The radius of the sensor. */
	real R;

	/** The view angle of the sensor. */
	real viewAngle;
	
	/** Half the view angle. */
	real delta;

	/** The view angle of an unit sensor (pixel). */
	real epsilon;

	/** Activates all sensors between gammaM and gammaP. The interval [gammaM, gammaP] is considered to be 
	fully ocluded by an object. */
	void activate(real gammaM, real gammaP);

	/** Computes the angle gamma corresponding to a point at dx, dy relative to the sensor center. 
	The value is normalized between (-Pi,Pi]. */
	real computeGamma(real dx, real dy){
		real alpha0 = atan2(dy, dx); //-Pi..Pi
		
		real gamma = alpha0 - (parent->alpha+relativeAlpha);
		
		while (gamma <= -M_PI) {
			gamma += 2*M_PI; 
		} 
		while (gamma > M_PI) {
			gamma -= 2*M_PI;
		}
	
		//gamma is now between (-Pi,Pi]
		return gamma;
	}

	//real g1,g2; //used for debugging

};

#endif //PROXIMITYSENSOR_H
