/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedAgentBase.h: interface for the ArticulatedAgentBase class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ARTICULATEDAGENTBASE_H
#define ARTICULATEDAGENTBASE_H

#include "ArticulatedComponent.h"
#include "ArticulatedLimb.h"
#include "Circle.h"
#include "CappedRectangle.h"
#include "Vector3.h"

class Simulator;

/** Base class for articulated agents. Should be specialized for either classical mechanics or 
quasistatic mechanics, for different types of bodies. */

class ArticulatedAgentBase : public ArticulatedComponent {
public:
	/** sets the body to NULL. */
	ArticulatedAgentBase(std::string label="");

	/** Destructor; deletes the limbs. */
	virtual ~ArticulatedAgentBase();

	/** Adds a limb to the agent and returns a reference to it. */
	ArticulatedLimb* addLimb();
	/** Adds a limb to the agent, sets the properties of the limb and returns a reference to it. 
	The limb is articulated at a point situated at l from the point of reference of the body, under
	and angle theta.*/
	ArticulatedLimb* addLimb(real l, real theta);


	virtual void registerPrimitives(Simulator* simulator);

	virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts);
	virtual void detectInternalContacts(GlobalContactInfoVector* contacts);
	virtual bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object);

	virtual void deleteContacts();

	void fillContactMatrix(ContactSolver* contactSolver, ContactInfo* contactInfo);

	/** Calls the draw methods for the body and the limbs. */
	virtual void draw(GUI *gui);

	virtual void setOutlineColor(Color color);
	virtual void setFillColor(Color color);

protected:
	/** Solves the system of equations m x=c; x is updated with the solution. */
	static void solveSystem(SymmetricMatrix3& m, const Vector3& c, Vector3& x);

	/** Solves the system of equations m x=c, given the precomputed determinant of m; x is updated 
	with the solution. */
	static void solveSystem(const SymmetricMatrix3& m, const real determinant, const Vector3& c, Vector3& x);
	
protected:
  /** A list containing the limbs. The list owns the limbs. */
  ArticulatedLimbPVector limbs;

	void deleteLimbs();
};

#endif //ARTICULATEDAGENTBASE_H
