/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedComponent.h: interface for the ArticulatedComponent class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ARTICULATEDCOMPONENT_H
#define ARTICULATEDCOMPONENT_H

#include "LinkContactInfo.h"
#include "ComposedPhysicalObject.h"
#include "SymmetricMatrix3.h"
#include "Vector3.h"

/** A component for articulated agents: it may represent the body of the agent, or (extended) an
articulated link. */

class ArticulatedComponent : public ComposedPhysicalObject  {
public:
	ArticulatedComponent(std::string label);

	/** Destructor; deletes K.*/
	virtual ~ArticulatedComponent();

	/** Parameters used by the Featherstone algorithm: */
	SymmetricMatrix3 IStar;
	SymmetricMatrix3 IStar0;
	Vector3 beta;
	Vector3 betaExternal;
	Vector3 betaStar;

	/** The acceleration of the object relative to the LRS, but expressed in the 
	local (object) reference system (ORS); used in the classical model	*/
	Vector2 aLocal;

	/** The velocity of the object relative to the LRS, but expressed in the 
	local (object) reference system (ORS); used in the quasistatic model	*/
	Vector2 vLocal;

	/** Computes the value of IStar0, based on the mass and moment of inertia of the object. */
	void computeIStar0();

	unsigned int nContacts;

	/** Link contact information used by the Featherstone algorithm. */
	LinkContactInfoPVector K;

	/** Link contact information used by the Featherstone algorithm. This list 
	contains objects, not pointers. */
	LinkContactInfoVector Q;

	/** Deletes K and its components. */
	void deleteK();
};

#endif //ARTICULATEDCOMPONENT_H
