/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// LinkContactInfo.h: interface for the LinkContactInfo class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LINKCONTACTINFO_H
#define LINKCONTACTINFO_H

#include "Vector3.h"
#include <vector>


class LinkContactInfo  {
public:
	LinkContactInfo() {}
	
	LinkContactInfo(const int& alpha, const Vector3& v) {
		this->alpha=alpha;
		this->v=v;
	}

	virtual ~LinkContactInfo(){}

	Vector3 v;
	real thetaFactor;
	int alpha;

	bool operator<( const LinkContactInfo& other) const { return this->alpha < other.alpha; }

	/** 
	// Compares two LinkContactInfo elements, based on the index alpha. 
	// Needed by the quicksort algorithm, for ordering the Q vector of 
	// ArticulatedAgentBase. 
	static int compare( const void *c1, const void *c2 ){
		return ((LinkContactInfo*)c1)->alpha-((LinkContactInfo*)c2)->alpha;
	}
	*/


};


/*

//These structs can be used by STL sort to sort LinkContactInfoLists

struct std::less<LinkContactInfo*>  {
	bool operator()(LinkContactInfo const* p1, LinkContactInfo const* p2){
		if(!p1)
			return true;
		if(!p2)
			return false;
		return p1->alpha < p2->alpha;
	}
};

struct std::greater<LinkContactInfo*> {
	bool operator()(LinkContactInfo const* p1, LinkContactInfo const* p2) {
		if(!p1)
			return false;
		if(!p2)
			return true;
		return p1->alpha > p2->alpha;
	}
};
*/


typedef std::vector<LinkContactInfo*> LinkContactInfoPVector;
typedef std::vector<LinkContactInfo> LinkContactInfoVector;


#endif //LINKCONTACTINFO_H
